@@ -130,6 +130,7 @@ struct GSLockPlugPrivate
         GtkWidget   *auth_face_image;
         GtkWidget   *auth_realname_label;
         GtkWidget   *auth_username_label;
+        GtkWidget   *auth_short_username_label;
         GtkWidget   *auth_password_entry;
         GtkWidget   *auth_capslock_label;
         GtkWidget   *auth_status_label;
@@ -169,6 +170,7 @@ struct _ResponseData
 
 enum {
         RESPONSE,
+        CLOSE,
         LAST_SIGNAL
 };
 
@@ -644,11 +646,29 @@ gs_lock_plug_get_property (GObject    *object,
         }
 }
 
+static void
+gs_lock_plug_close (GSLockPlug *plug)
+{
+        /* Synthesize delete_event to close dialog. */
+  
+        GtkWidget *widget = GTK_WIDGET (plug);
+        GdkEvent  *event;
+
+        event = gdk_event_new (GDK_DELETE);
+  
+        event->any.window = g_object_ref (widget->window);
+        event->any.send_event = TRUE;
+  
+        gtk_main_do_event (event);
+        gdk_event_free (event);
+}
+
 static void
 gs_lock_plug_class_init (GSLockPlugClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
         GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+        GtkBindingSet  *binding_set;
 
         parent_class = g_type_class_peek_parent (klass);
 
@@ -661,8 +681,9 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
         widget_class->hide         = gs_lock_plug_hide;
         widget_class->size_request = gs_lock_plug_size_request;
 
-        g_type_class_add_private (klass, sizeof (GSLockPlugPrivate));
+        klass->close = gs_lock_plug_close;
 
+        g_type_class_add_private (klass, sizeof (GSLockPlugPrivate));
 
         lock_plug_signals [RESPONSE] = g_signal_new ("response",
                                                      G_OBJECT_CLASS_TYPE (klass),
@@ -672,6 +693,13 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                      g_cclosure_marshal_VOID__INT,
                                                      G_TYPE_NONE, 1,
                                                      G_TYPE_INT);
+        lock_plug_signals [CLOSE] = g_signal_new ("close",
+                                                  G_OBJECT_CLASS_TYPE (klass),
+                                                  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                                  G_STRUCT_OFFSET (GSLockPlugClass, close),
+                                                  NULL, NULL,
+                                                  g_cclosure_marshal_VOID__VOID,
+                                                  G_TYPE_NONE, 0);
 
         g_object_class_install_property (object_class,
                                          PROP_LOGOUT_ENABLED,
@@ -694,6 +722,11 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                                NULL,
                                                                FALSE,
                                                                G_PARAM_READWRITE));
+
+        binding_set = gtk_binding_set_by_class (klass);
+  
+        gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0,
+                                      "close", 0);
 }
 
 static gboolean
@@ -1217,6 +1250,22 @@ get_user_on_host_name (void)
         return str;
 }
 
+static char *
+get_user_name (void)
+{
+        const char *name;
+        char       *utf8_name;
+
+        name = g_get_user_name ();
+
+        utf8_name = NULL;
+        if (name != NULL) {
+                utf8_name = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
+        }
+
+        return utf8_name;
+}
+
 static gboolean
 check_user_file (const gchar *filename,
 		 uid_t        user,
@@ -1598,6 +1647,7 @@ load_theme (GSLockPlug *plug)
         plug->priv->auth_action_area = glade_xml_get_widget (xml, "auth-action-area");
         plug->priv->auth_realname_label = glade_xml_get_widget (xml, "auth-realname-label");
         plug->priv->auth_username_label = glade_xml_get_widget (xml, "auth-username-label");
+        plug->priv->auth_short_username_label = glade_xml_get_widget (xml, "auth-short-username-label");
         plug->priv->auth_password_entry = glade_xml_get_widget (xml, "auth-password-entry");
         plug->priv->auth_capslock_label = glade_xml_get_widget (xml, "auth-capslock-label");
         plug->priv->auth_status_label = glade_xml_get_widget (xml, "auth-status-label");
@@ -1614,6 +1664,16 @@ load_theme (GSLockPlug *plug)
         return TRUE;
 }
 
+static int
+delete_handler (GSLockPlug  *plug,
+                GdkEventAny *event,
+                gpointer     data)
+{
+        gs_lock_plug_response (plug, GS_LOCK_PLUG_RESPONSE_CANCEL);
+
+        return TRUE; /* Do not destroy */
+}
+
 #define INVISIBLE_CHAR_DEFAULT       '*'
 #define INVISIBLE_CHAR_BLACK_CIRCLE  0x25cf
 #define INVISIBLE_CHAR_WHITE_BULLET  0x25e6
@@ -1669,6 +1729,18 @@ gs_lock_plug_init (GSLockPlug *plug)
                 g_free (str);
         }
 
+        if (plug->priv->auth_short_username_label != NULL) {
+                char       *str;
+                char       *name;
+
+                /* FIXME should let the theme determine the font size */
+                name = get_user_name ();
+                str = g_strdup_printf ("<span size=\"x-large\">%s</span>", name);
+                g_free (name);
+                gtk_label_set_markup (GTK_LABEL (plug->priv->auth_short_username_label), str);
+                g_free (str);
+        }
+
         if (plug->priv->auth_realname_label != NULL) {
                 char       *str;
                 char       *name;
@@ -1736,6 +1808,8 @@ gs_lock_plug_init (GSLockPlug *plug)
                 set_face_image (plug);
         }
 
+        g_signal_connect (plug, "delete_event", G_CALLBACK (delete_handler), NULL);
+
         g_idle_add ((GSourceFunc)setup_treeview_idle, plug);
 
         profile_end ("end", NULL);
