@@ -43,7 +43,7 @@ static void gs_manager_finalize   (GObject        *object);
 struct GSManagerPrivate
 {
         GSList      *windows;
-        GSList      *jobs;
+        GHashTable  *jobs;
 
         /* Policy */
         glong        lock_timeout;
@@ -53,7 +53,7 @@ struct GSManagerPrivate
         guint        lock_enabled : 1;
         guint        logout_enabled : 1;
         guint        user_switch_enabled : 1;
-        guint        throttle_enabled : 1;
+        guint        throttled : 1;
 
         char        *logout_command;
 
@@ -61,6 +61,7 @@ struct GSManagerPrivate
         guint        active : 1;
         guint        lock_active : 1;
 
+        guint        fading : 1;
         guint        dialog_up : 1;
 
         guint        lock_timeout_id;
@@ -70,11 +71,15 @@ struct GSManagerPrivate
 
         GSList      *themes;
         GSSaverMode  saver_mode;
+        GSFade      *fade;
+        guint        unfade_idle_id;
 };
 
 enum {
         ACTIVATED,
         DEACTIVATED,
+        AUTH_REQUEST_BEGIN,
+        AUTH_REQUEST_END,
         LAST_SIGNAL
 };
 
@@ -88,7 +93,7 @@ enum {
         PROP_LOGOUT_TIMEOUT,
         PROP_LOGOUT_COMMAND,
         PROP_ACTIVE,
-        PROP_THROTTLE_ENABLED,
+        PROP_THROTTLED,
 };
 
 #define FADE_TIMEOUT 1000
@@ -98,6 +103,131 @@ static guint         signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSManager, gs_manager, G_TYPE_OBJECT)
 
+static void
+manager_add_job_for_window (GSManager *manager,
+                            GSWindow  *window,
+                            GSJob     *job)
+{
+        if (manager->priv->jobs == NULL) {
+                return;
+        }
+
+        g_hash_table_insert (manager->priv->jobs, window, job);
+}
+
+static const char *
+select_theme (GSManager *manager)
+{
+        const char *theme = NULL;
+
+        g_return_val_if_fail (manager != NULL, NULL);
+        g_return_val_if_fail (GS_IS_MANAGER (manager), NULL);
+
+        if (manager->priv->saver_mode == GS_MODE_BLANK_ONLY) {
+                return NULL;
+        }
+
+        if (manager->priv->themes) {
+                int number = 0;
+
+                if (manager->priv->saver_mode == GS_MODE_RANDOM) {
+                        g_random_set_seed (time (NULL));
+                        number = g_random_int_range (0, g_slist_length (manager->priv->themes));
+                }
+                theme = g_slist_nth_data (manager->priv->themes, number);
+        }
+
+        return theme;
+}
+
+static void
+cycle_job (GSWindow  *window,
+           GSJob     *job,
+           GSManager *manager)
+{
+        const char *theme;
+
+        theme = select_theme (manager);
+
+        gs_job_stop (job);
+        gs_job_set_theme (job, theme, NULL);
+        gs_job_start (job);
+}
+
+static void
+manager_cycle_jobs (GSManager *manager)
+{
+        if (manager->priv->jobs != NULL) {
+                g_hash_table_foreach (manager->priv->jobs, (GHFunc) cycle_job, manager);
+        }
+}
+
+static void
+throttle_job (GSWindow  *window,
+              GSJob     *job,
+              GSManager *manager)
+{
+        if (manager->priv->throttled) {
+                gs_job_stop (job);
+        } else {
+                gs_job_start (job);
+        }
+}
+
+static void
+manager_throttle_jobs (GSManager *manager)
+{
+        if (manager->priv->jobs != NULL) {
+                g_hash_table_foreach (manager->priv->jobs, (GHFunc) throttle_job, manager);
+        }
+}
+
+static void
+resume_job (GSWindow  *window,
+            GSJob     *job,
+            GSManager *manager)
+{
+        if (gs_job_is_running (job)) {
+                gs_job_suspend (job, FALSE);
+        } else {
+                gs_job_start (job);
+        }
+}
+
+static void
+manager_resume_jobs (GSManager *manager)
+{
+        if (manager->priv->jobs != NULL) {
+                g_hash_table_foreach (manager->priv->jobs, (GHFunc) resume_job, manager);
+        }
+}
+
+static void
+suspend_job (GSWindow  *window,
+             GSJob     *job,
+             GSManager *manager)
+{
+        gs_job_suspend (job, TRUE);
+}
+
+static void
+manager_suspend_jobs (GSManager *manager)
+{
+        if (manager->priv->jobs != NULL) {
+                g_hash_table_foreach (manager->priv->jobs, (GHFunc) suspend_job, manager);
+        }
+}
+
+static void
+manager_stop_jobs (GSManager *manager)
+{
+        if (manager->priv->jobs != NULL) {
+                g_hash_table_destroy (manager->priv->jobs);
+
+        }
+        manager->priv->jobs = NULL;
+}
+
 void
 gs_manager_set_mode (GSManager  *manager,
                      GSSaverMode mode)
@@ -127,25 +257,19 @@ gs_manager_set_themes (GSManager *manager,
 }
 
 void
-gs_manager_set_throttle_enabled (GSManager *manager,
-                                 gboolean   throttle_enabled)
+gs_manager_set_throttled (GSManager *manager,
+                          gboolean   throttled)
 {
         g_return_if_fail (GS_IS_MANAGER (manager));
 
-        if (manager->priv->throttle_enabled != throttle_enabled) {
+        if (manager->priv->throttled != throttled) {
                 GSList *l;
 
-                manager->priv->throttle_enabled = throttle_enabled;
+                manager->priv->throttled = throttled;
 
                 if (! manager->priv->dialog_up) {
 
-                        for (l = manager->priv->jobs; l; l = l->next) {
-                                if (throttle_enabled) {
-                                        gs_job_stop (GS_JOB (l->data));
-                                } else {
-                                        gs_job_start (GS_JOB (l->data));
-                                }
-                        }
+                        manager_throttle_jobs (manager);
 
                         for (l = manager->priv->windows; l; l = l->next) {
                                 gs_window_clear (l->data);
@@ -160,6 +284,8 @@ gs_manager_set_lock_active (GSManager *manager,
 {
         g_return_if_fail (GS_IS_MANAGER (manager));
 
+        gs_debug ("Setting lock active: %d", lock_active);
+
         if (manager->priv->lock_active != lock_active) {
                 GSList *l;
 
@@ -216,8 +342,9 @@ gs_manager_set_user_switch_enabled (GSManager *manager,
 static gboolean
 activate_lock_timeout (GSManager *manager)
 {
-        if (manager->priv->lock_enabled)
+        if (manager->priv->lock_enabled) {
                 gs_manager_set_lock_active (manager, TRUE);
+        }
 
         manager->priv->lock_timeout_id = 0;
 
@@ -300,38 +427,9 @@ gs_manager_set_logout_command (GSManager  *manager,
         }
 }
 
-static const char *
-select_theme (GSManager *manager)
-{
-        const char *theme = NULL;
-
-        g_return_val_if_fail (manager != NULL, NULL);
-        g_return_val_if_fail (GS_IS_MANAGER (manager), NULL);
-
-        if (manager->priv->saver_mode == GS_MODE_BLANK_ONLY) {
-                return NULL;
-        }
-
-        if (manager->priv->themes) {
-                int number = 0;
-
-                if (manager->priv->saver_mode == GS_MODE_RANDOM) {
-                        g_random_set_seed (time (NULL));
-                        number = g_random_int_range (0, g_slist_length (manager->priv->themes));
-                }
-                theme = g_slist_nth_data (manager->priv->themes, number);
-        }
-
-        return theme;
-}
-
 gboolean
 gs_manager_cycle (GSManager *manager)
 {
-        GSList     *l;
-        const char *theme;
-        gboolean    success = FALSE;
-
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
@@ -343,23 +441,13 @@ gs_manager_cycle (GSManager *manager)
                 return FALSE;
         }
 
-        if (manager->priv->throttle_enabled) {
+        if (manager->priv->throttled) {
                 return FALSE;
         }
 
-        theme = select_theme (manager);
+        manager_cycle_jobs (manager);
 
-        for (l = manager->priv->jobs; l; l = l->next) {
-                gs_job_stop (GS_JOB (l->data));
-                gs_job_set_theme (GS_JOB (l->data), theme, NULL);
-
-                /* this success flag is kinda bogus */
-                if (gs_job_start (GS_JOB (l->data))) {
-                        success = TRUE;
-                }
-        }
-
-        return success;
+        return TRUE;
 }
 
 static gboolean
@@ -432,8 +520,8 @@ gs_manager_set_property (GObject            *object,
         self = GS_MANAGER (object);
 
         switch (prop_id) {
-        case PROP_THROTTLE_ENABLED:
-                gs_manager_set_throttle_enabled (self, g_value_get_boolean (value));
+        case PROP_THROTTLED:
+                gs_manager_set_throttled (self, g_value_get_boolean (value));
                 break;
         case PROP_LOCK_ENABLED:
                 gs_manager_set_lock_enabled (self, g_value_get_boolean (value));
@@ -473,8 +561,8 @@ gs_manager_get_property (GObject            *object,
         self = GS_MANAGER (object);
 
         switch (prop_id) {
-        case PROP_THROTTLE_ENABLED:
-                g_value_set_boolean (value, self->priv->throttle_enabled);
+        case PROP_THROTTLED:
+                g_value_set_boolean (value, self->priv->throttled);
                 break;
         case PROP_LOCK_ENABLED:
                 g_value_set_boolean (value, self->priv->lock_enabled);
@@ -536,6 +624,26 @@ gs_manager_class_init (GSManagerClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
+        signals [AUTH_REQUEST_BEGIN] =
+                g_signal_new ("auth-request-begin",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GSManagerClass, auth_request_begin),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+        signals [AUTH_REQUEST_END] =
+                g_signal_new ("auth-request-end",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GSManagerClass, auth_request_end),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
 
         g_object_class_install_property (object_class,
                                          PROP_ACTIVE,
@@ -601,8 +709,8 @@ gs_manager_class_init (GSManagerClass *klass)
                                                             300000,
                                                             G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
-                                         PROP_THROTTLE_ENABLED,
-                                         g_param_spec_boolean ("throttle-enabled",
+                                         PROP_THROTTLED,
+                                         g_param_spec_boolean ("throttled",
                                                                NULL,
                                                                NULL,
                                                                TRUE,
@@ -615,6 +723,8 @@ static void
 gs_manager_init (GSManager *manager)
 {
         manager->priv = GS_MANAGER_GET_PRIVATE (manager);
+
+        manager->priv->fade = gs_fade_new ();
 }
 
 static void
@@ -643,12 +753,7 @@ gs_manager_finalize (GObject *object)
 
         gs_grab_release_keyboard_and_mouse ();
 
-        for (l = manager->priv->jobs; l; l = l->next) {
-                gs_job_stop (GS_JOB (l->data));
-                g_object_unref (l->data);
-        }
-        g_slist_free (manager->priv->jobs);
-        manager->priv->jobs = NULL;
+        manager_stop_jobs (manager);
 
         for (l = manager->priv->windows; l; l = l->next) {
                 gs_window_destroy (l->data);
@@ -660,6 +765,8 @@ gs_manager_finalize (GObject *object)
         manager->priv->activate_time = 0;
         manager->priv->lock_enabled = FALSE;
 
+        g_object_unref (manager->priv->fade);
+
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -697,13 +804,19 @@ window_dialog_up_cb (GSWindow  *window,
 
         gs_debug ("Handling dialog up");
 
+        g_signal_emit (manager, signals [AUTH_REQUEST_BEGIN], 0);
+
         manager->priv->dialog_up = TRUE;
 
-        /* Grab keyboard so dialog can be used */
+        /* Move keyboard and mouse grabs so dialog can be used */
         gs_grab_window (gs_window_get_gdk_window (window),
                         gs_window_get_screen (window),
                         FALSE);
 
+        /* Release the pointer grab while dialog is up so that
+           the dialog can be used.  We'll regrab it when the dialog goes down. */
+        gs_grab_release_mouse ();
+
         /* Make all other windows insensitive so we don't get events */
         for (l = manager->priv->windows; l; l = l->next) {
                 if (l->data != window) {
@@ -711,12 +824,10 @@ window_dialog_up_cb (GSWindow  *window,
                 }
         }
 
-        if (! manager->priv->throttle_enabled) {
+        if (! manager->priv->throttled) {
                 gs_debug ("Suspending jobs");
 
-                for (l = manager->priv->jobs; l; l = l->next) {
-                        gs_job_suspend (GS_JOB (l->data), TRUE);
-                }
+                manager_suspend_jobs (manager);
         }
 }
 
@@ -729,22 +840,116 @@ window_dialog_down_cb (GSWindow  *window,
         g_return_if_fail (manager != NULL);
         g_return_if_fail (GS_IS_MANAGER (manager));
 
+        /* Regrab the mouse */
+        gs_grab_window (gs_window_get_gdk_window (window),
+                        gs_window_get_screen (window),
+                        FALSE);
+
         /* Make all windows sensitive so we get events */
         for (l = manager->priv->windows; l; l = l->next) {
                 gtk_widget_set_sensitive (GTK_WIDGET (l->data), TRUE);
         }
 
-        if (! manager->priv->throttle_enabled) {
-                for (l = manager->priv->jobs; l; l = l->next) {
-                        if (gs_job_is_running (GS_JOB (l->data))) {
-                                gs_job_suspend (GS_JOB (l->data), FALSE);
-                        } else {
-                                gs_job_start (GS_JOB (l->data));
-                        }
-                }
+        if (! manager->priv->throttled) {
+                manager_resume_jobs (manager);
         }
 
         manager->priv->dialog_up = FALSE;
+
+        g_signal_emit (manager, signals [AUTH_REQUEST_END], 0);
+}
+
+static void
+manager_maybe_start_job_for_window (GSManager *manager,
+                                    GSWindow  *window)
+{
+        GSJob *job;
+
+        if (manager->priv->jobs == NULL) {
+                return;
+        }
+
+        job = g_hash_table_lookup (manager->priv->jobs, window);
+
+        if (job == NULL) {
+                gs_debug ("Job not found for window");
+                return;
+        }
+
+        if (! manager->priv->dialog_up) {
+                if (! manager->priv->throttled) {
+                        if (! gs_job_is_running (job)) {
+                                gs_debug ("Starting job for window");
+                                gs_job_start (job);
+                        }
+                }
+        }
+}
+
+static gboolean
+manager_maybe_grab_window (GSManager *manager,
+                           GSWindow  *window)
+{
+        GdkDisplay *display;
+        GdkScreen  *screen;
+        int         monitor;
+        int         x, y;
+        gboolean    grabbed;
+
+        display = gdk_display_get_default ();
+        gdk_display_get_pointer (display, &screen, &x, &y, NULL);
+        monitor = gdk_screen_get_monitor_at_point (screen, x, y);
+
+        gdk_flush ();
+        grabbed = FALSE;
+        if (gs_window_get_screen (window) == screen
+            && gs_window_get_monitor (window) == monitor) {
+                gs_grab_window (gs_window_get_gdk_window (window),
+                                gs_window_get_screen (window),
+                                FALSE);
+                grabbed = TRUE;
+        }
+
+        return grabbed;
+}
+
+static void
+window_grab_broken_cb (GSWindow           *window,
+                       GdkEventGrabBroken *event,
+                       GSManager          *manager)
+{
+        gs_debug ("GRAB BROKEN!");
+        if (event->keyboard) {
+                gs_grab_keyboard_reset ();
+        } else {
+                gs_grab_mouse_reset ();
+        }
+}
+
+static gboolean
+unfade_idle (GSManager *manager)
+{
+        gs_debug ("resetting fade");
+        gs_fade_reset (manager->priv->fade);
+        manager->priv->unfade_idle_id = 0;
+        return FALSE;
+}
+
+
+static void
+remove_unfade_idle (GSManager *manager)
+{
+        if (manager->priv->unfade_idle_id > 0) {
+                g_source_remove (manager->priv->unfade_idle_id);
+                manager->priv->unfade_idle_id = 0;
+        }
+}
+
+static void
+add_unfade_idle (GSManager *manager)
+{
+        remove_unfade_idle (manager);
+        manager->priv->unfade_idle_id = g_idle_add ((GSourceFunc)unfade_idle, manager);
 }
 
 static gboolean
@@ -752,58 +957,45 @@ window_map_event_cb (GSWindow  *window,
                      GdkEvent  *event,
                      GSManager *manager)
 {
-        GdkDisplay *display;
-        GdkScreen  *screen;
-        int         monitor;
-        int         x, y;
+        gs_debug ("Handling window map_event event");
 
-        g_return_val_if_fail (manager != NULL, FALSE);
-        g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
-        g_return_val_if_fail (window != NULL, FALSE);
-        g_return_val_if_fail (GS_IS_WINDOW (window), FALSE);
+        manager_maybe_grab_window (manager, window);
 
-        display = gdk_display_get_default ();
-        gdk_display_get_pointer (display, &screen, &x, &y, NULL);
-        monitor = gdk_screen_get_monitor_at_point (screen, x, y);
-
-        if (gs_window_get_screen (window) == screen
-            && gs_window_get_monitor (window) == monitor) {
-                gs_grab_window (gs_window_get_gdk_window (window),
-                                gs_window_get_screen (window),
-                                FALSE);
-        }
+        manager_maybe_start_job_for_window (manager, window);
 
         return FALSE;
 }
 
 static void
-window_show_cb (GSWindow  *window,
-                GSManager *manager)
+window_map_cb (GSWindow  *window,
+               GSManager *manager)
+{
+        gs_debug ("Handling window map event");
+}
+
+static void
+window_unmap_cb (GSWindow  *window,
+                 GSManager *manager)
+{
+        gs_debug ("window unmapped!");
+}
+
+static void
+manager_show_window (GSManager *manager,
+                     GSWindow  *window)
 {
         GSJob      *job;
         const char *theme;
 
-        g_return_if_fail (manager != NULL);
-        g_return_if_fail (GS_IS_MANAGER (manager));
-        g_return_if_fail (window != NULL);
-        g_return_if_fail (GS_IS_WINDOW (window));
-
         job = gs_job_new_for_widget (GTK_WIDGET (window));
 
         theme = select_theme (manager);
         gs_job_set_theme (job, theme, NULL);
 
-        if (! manager->priv->throttle_enabled) {
-                gs_job_start (job);
-        }
-
-        manager->priv->jobs = g_slist_append (manager->priv->jobs, job);
+        manager_add_job_for_window (manager, window, job);
 
         manager->priv->activate_time = time (NULL);
 
-        /* Set lock state to off initially.  It may be enabled by activate_lock_timout() */
-        gs_manager_set_lock_active (manager, FALSE);
-
         if (manager->priv->lock_timeout >= 0) {
                 remove_lock_timer (manager);
                 add_lock_timer (manager, manager->priv->lock_timeout);
@@ -814,10 +1006,26 @@ window_show_cb (GSWindow  *window,
                 add_cycle_timer (manager, manager->priv->cycle_timeout);
         }
 
+        add_unfade_idle (manager);
+
         /* FIXME: only emit signal once */
         g_signal_emit (manager, signals [ACTIVATED], 0);
 }
 
+static void
+window_show_cb (GSWindow  *window,
+                GSManager *manager)
+{
+
+        g_return_if_fail (manager != NULL);
+        g_return_if_fail (GS_IS_MANAGER (manager));
+        g_return_if_fail (window != NULL);
+        g_return_if_fail (GS_IS_WINDOW (window));
+
+        gs_debug ("Handling window show");
+        manager_show_window (manager, window);
+}
+
 static void
 disconnect_window_signals (GSManager *manager,
                            GSWindow  *window)
@@ -826,7 +1034,10 @@ disconnect_window_signals (GSManager *manager,
         g_signal_handlers_disconnect_by_func (window, window_dialog_up_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_dialog_down_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_show_cb, manager);
+        g_signal_handlers_disconnect_by_func (window, window_map_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_event_cb, manager);
+        g_signal_handlers_disconnect_by_func (window, window_unmap_cb, manager);
+        g_signal_handlers_disconnect_by_func (window, window_grab_broken_cb, manager);
 }
 
 static void
@@ -849,9 +1060,15 @@ connect_window_signals (GSManager *manager,
         g_signal_connect_object (window, "dialog-down",
                                  G_CALLBACK (window_dialog_down_cb), manager, 0);
         g_signal_connect_object (window, "show",
-                                 G_CALLBACK (window_show_cb), manager, 0);
+                                 G_CALLBACK (window_show_cb), manager, G_CONNECT_AFTER);
+        g_signal_connect_object (window, "map",
+                                 G_CALLBACK (window_map_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "map_event",
-                                 G_CALLBACK (window_map_event_cb), manager, 0);
+                                 G_CALLBACK (window_map_event_cb), manager, G_CONNECT_AFTER);
+        g_signal_connect_object (window, "unmap",
+                                 G_CALLBACK (window_unmap_cb), manager, G_CONNECT_AFTER);
+        g_signal_connect_object (window, "grab_broken_event",
+                                 G_CALLBACK (window_grab_broken_cb), manager, G_CONNECT_AFTER);
 }
 
 static void
@@ -932,6 +1149,26 @@ show_windows (GSList *windows)
         }
 }
 
+static void
+remove_job (GSJob *job)
+{
+        if (job == NULL) {
+                return;
+        }
+
+        gs_job_stop (job);
+        g_object_unref (job);
+}
+
+static void
+fade_done_cb (GSFade    *fade,
+              GSManager *manager)
+{
+        gs_debug ("fade completed, showing windows");
+        show_windows (manager->priv->windows);
+        manager->priv->fading = FALSE;
+}
+
 static gboolean
 gs_manager_activate (GSManager *manager)
 {
@@ -956,23 +1193,26 @@ gs_manager_activate (GSManager *manager)
                 return FALSE;
         }
 
-        if (! manager->priv->windows)
+        if (! manager->priv->windows) {
                 gs_manager_create (GS_MANAGER (manager));
+        }
+
+        manager->priv->jobs = g_hash_table_new_full (g_direct_hash,
+                                                     g_direct_equal,
+                                                     NULL,
+                                                     (GDestroyNotify)remove_job);
 
         manager->priv->active = TRUE;
 
         /* fade to black and show windows */
         do_fade = TRUE;
         if (do_fade) {
-                GSFade *fade;
-
-                fade = gs_fade_new ();
-                gs_fade_set_timeout (fade, FADE_TIMEOUT);
-                gs_fade_now (fade);
-                show_windows (manager->priv->windows);
-                g_usleep (500000);
-                gs_fade_reset (fade);
-                g_object_unref (fade);
+                manager->priv->fading = TRUE;
+                gs_debug ("fading out");
+                gs_fade_async (manager->priv->fade,
+                               FADE_TIMEOUT,
+                               (GSFadeDoneFunc)fade_done_cb,
+                               manager);
         } else {
                 show_windows (manager->priv->windows);
         }
@@ -997,12 +1237,7 @@ gs_manager_deactivate (GSManager *manager)
 
         gs_grab_release_keyboard_and_mouse ();
 
-        for (l = manager->priv->jobs; l; l = l->next) {
-                gs_job_stop (GS_JOB (l->data));
-                g_object_unref (l->data);
-        }
-        g_slist_free (manager->priv->jobs);
-        manager->priv->jobs = NULL;
+        manager_stop_jobs (manager);
 
         for (l = manager->priv->windows; l; l = l->next) {
                 gs_window_destroy (l->data);
@@ -1013,7 +1248,7 @@ gs_manager_deactivate (GSManager *manager)
         manager->priv->active = FALSE;
         manager->priv->activate_time = 0;
         manager->priv->lock_active = FALSE;
-        manager->priv->throttle_enabled = FALSE;
+        manager->priv->throttled = FALSE;
 
         return TRUE;
 }
@@ -1054,9 +1289,15 @@ gs_manager_request_unlock (GSManager *manager)
         g_return_if_fail (GS_IS_MANAGER (manager));
 
         if (! manager->priv->active) {
+                gs_debug ("Request unlock but manager is not active");
                 return;
         }
 
+        if (manager->priv->fading) {
+                gs_debug ("Request unlock so finishing fade");
+                gs_fade_finish (manager->priv->fade);
+        }
+
         /* Find the screen that contains the pointer */
         display = gdk_display_get_default ();
         gdk_display_get_pointer (display, &screen, NULL, NULL, NULL);
@@ -1064,7 +1305,9 @@ gs_manager_request_unlock (GSManager *manager)
 
         /* Find the gs-window that is on that screen */
         for (l = manager->priv->windows; l; l = l->next) {
-                int num = gdk_screen_get_number (GTK_WINDOW (l->data)->screen);
+                int num;
+
+                num = gdk_screen_get_number (GTK_WINDOW (l->data)->screen);
                 if (num == screen_num) {
                         gs_window_request_unlock (GS_WINDOW (l->data));
                         break;
